#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File:
#   alertdsp.py
#
# Descripion:
#   Snort alert displaying as treemaps.
#
# Author:
#   Jorge D. Ortiz Fuentes <jdortiz@gmail.com>
# Copyright (c) 2007 Jorge D. Ortiz Fuentes <jdortiz@gmail.com>
#
# License:
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Versions:
#   2007-11-10. First version
#

import alertgrp
import wx
from wx.lib.wordwrap import wordwrap
import re
import sys

class AlertDspFrm (wx.Frame):
    def __init__(self, title, pos, size):
        wx.Frame.__init__(self, None, -1, title, pos, size)
        # regular expresions to escape especial characters
        self.dollar = re.compile('\$')
        self.min_size = 4 # Minimum size that a button must have to be drawn
        self.alerts = None
        self.filter = []
        self.title = title
        # ID creation
        ID_M_OPEN = wx.NewId()
        ID_M_EXIT = wx.NewId()
        ID_M_ABOUT = wx.NewId()
        ID_CHOICE = wx.NewId()
        self.ID_UP = wx.NewId()

        panel = wx.Panel(self)

        # Create the tool bar with a choice control fro the grouping field
        self.toolbar = self.CreateToolBar(wx.TB_HORIZONTAL)
        tsize = (24, 24)
        self.toolbar.SetToolBitmapSize(tsize)
        up_bmp = wx.ArtProvider.GetBitmap(wx.ART_GO_UP, wx.ART_TOOLBAR, tsize)
        self.toolbar.AddLabelTool(self.ID_UP, "UP", up_bmp, shortHelp="Up")
        self.toolbar.EnableTool(self.ID_UP, False)
        self.Bind(wx.EVT_MENU, self.onUp, id=self.ID_UP) 
        self.toolbar.AddSeparator
        self.choice = wx.Choice(self.toolbar, ID_CHOICE, choices=alertgrp.fields)
        self.choice.Select(alertgrp.fields.index('alert'))
        self.choice.SetToolTipString("Field to group by")
        self.toolbar.AddControl(self.choice)
        self.Bind(wx.EVT_CHOICE, self.onChoice)
        self.toolbar.Bind(wx.EVT_LEAVE_WINDOW, self.onLeave)

        # Create the menu bar
        menuFile = wx.Menu()
        menuFile.Append(ID_M_OPEN, "&Open...", "Open a Snort alert file")
        menuFile.AppendSeparator()
        menuFile.Append(ID_M_EXIT, "E&xit", "Exit the program")
        menuHelp = wx.Menu()
        menuHelp.Append(ID_M_ABOUT, "&About...", "Credits")
        menuBar = wx.MenuBar()
        menuBar.Append(menuFile, "&File")
        menuBar.Append(menuHelp, "&Help")
        self.SetMenuBar(menuBar)
        # And the corresponding events
        self.Bind(wx.EVT_MENU, self.onOpen, id=ID_M_OPEN)
        self.Bind(wx.EVT_MENU, self.onQuit, id=ID_M_EXIT)
        self.Bind(wx.EVT_MENU, self.onAbout, id=ID_M_ABOUT)

        # Status bar
        self.CreateStatusBar()
        self.SetStatusText("Filter: None")

        # create the window where the painting happens
        self.window = wx.Window(panel, -1)
        container = wx.BoxSizer(wx.HORIZONTAL)
        container.Add(self.window, 1, wx.EXPAND)
        panel.SetSizerAndFit(container)
        self.button_id = {}
        self.window.Bind(wx.EVT_SIZE, self.onSize)
        
        # Create the pop-up menu to select the next grouping field
        self.popupmenu = wx.Menu()
        for l in alertgrp.fields:
            item = self.popupmenu.Append(-1, l)
            self.popupmenu.Bind(wx.EVT_MENU, self.onPopupItemSelected, item)

        # paint treemap
        self.displayTreemap('alert')


    def setData (self, filename):
        if filename != None:
            self.alerts = alertgrp.AlertSet(filename)
            # Reset parameters
            self.filter = []
            self.updateChoice()
            self.choice.Select(self.choice.GetItems().index('alert'))
            self.toolbar.EnableTool(self.ID_UP, False)
            self.updateStatusBar()
            # Set titlebar
            self.SetTitle(filename + ' -- ' + self.title)

        # paint treemap
        self.displayTreemap('alert')


    def onSize(self, event):
        """Deal with (re)sizing events"""
        self.displayTreemap(self.choice.GetItems()[self.choice.GetSelection()])


    def onOpen (self, event):
        """Deal with the menu File->Open"""
        dlg = wx.FileDialog(self, message="Open Snort alert file",
                            defaultDir="", defaultFile="", wildcard="*.*",
                            style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.setData(dlg.GetPath())
        dlg.Destroy()


    def onQuit (self, event):
        """Exit the application when requested through the menu"""
        self.Close()


    def onChoice (self, event):
        """Deal with the changes of the grouping field"""
        self.displayTreemap(alertgrp.fields[event.Selection])


    def onAbout (self, event):
        """Display the credits when requested"""
#        wx.MessageBox("RaDaJo Alert Treemap Display\n\
#Copyright (c) Jorge D. Ortiz Fuentes <jdortiz@gmail.com> 2007",
#                      "About Alert Display",
#                      wx.OK | wx.ICON_INFORMATION, self)
        licenseText = """\
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>"""
        info = wx.AboutDialogInfo()
        info.Name = "About Alert Display"
        info.Version = "1.0"
        info.Copyright = "Copyright (C) 2007 RaDaJo"
        info.Description = "Snort Treemap Alert Display"
        info.WebSite = ("http://www.radajo.com", "RaDaJo web page")
        info.Developers = [ "Jorge Ortiz <jdortiz@gmail.com>"]

        info.License = wordwrap(licenseText, 500, wx.ClientDC(self))

        wx.AboutBox(info)


    def onSelect (self, event):
        self.toolbar.EnableTool(self.ID_UP, True)
        id = event.GetId()
        for k,v in self.button_id.items():
            if v == id:
                # This is set in a tuple and appended to a list to allow going back
                # escape field so especial characters aren't used in regexps
                value = self.dollar.sub('\$', k)
                self.filter.append((self.choice.GetItems()[self.choice.GetSelection()], value))
                self.updateStatusBar()

                # Disable the choice that cannot be used
                self.updateChoice()

                # Disable the grouping fields that cannot be used
                dis_opt = []
                for i in self.filter:
                    dis_opt.append(i[0])
                for o in self.popupmenu.GetMenuItems():
                    o.Enable(True)
                    if o.GetLabel() in dis_opt:
                        o.Enable(False)

                # Pop up the menu to select the next grouping field
                pos = event.GetEventObject().GetPosition()
                size = event.GetEventObject().GetSize()
                self.window.PopupMenu(self.popupmenu,
                                      wx.Point(pos.x + size.x/2,
                                               pos.y + size.y/2))
                break


    def onPopupItemSelected (self, event):
        item = self.popupmenu.FindItemById(event.GetId())
        field = item.GetText()
        self.choice.Select(self.choice.GetItems().index(field))
        self.displayTreemap(field)


    def onUp (self, event):
        f = self.filter.pop()
        if len(self.filter) == 0:
            self.toolbar.EnableTool(self.ID_UP, False)
        self.updateChoice()
        self.updateStatusBar()
        self.choice.Select(self.choice.GetItems().index(f[0]))
        self.displayTreemap(self.choice.GetItems()[self.choice.GetSelection()])
        

    def onLeave (self, event):
        """Update the status bar to compensate for the longhelp being displayed
and deleting the text."""
        self.updateStatusBar()


    def displayTreemap (self, field):
        """Paint the treemap of the data grouped by the selected field"""
        # clean up
        if len(self.button_id) > 0:
            self.window.DestroyChildren()

        # create the new buttonswidgets
        size = self.window.GetSize()
        if self.alerts != None:
            # prepare filters
            if len(self.filter) > 0:
                filter = {}
                for i in self.filter:
                    filter[i[0]] = i[1]
                list = self.alerts.group(field, filter)
            else:
                list = self.alerts.group(field)
            self.button_id = {}
            if len(list) > 0:
                self.divideDisplayArea(self.window, wx.Size(0, 0), size, list)


    def divideDisplayArea (self, window, top_left, bottom_right, list):
        """Use the treemaps algorithm as explained by the SpaceMonger authors. Thanks!"""
        l1 = {}
        l2 = {}
        l1sum = l2sum = 0
        li = [(v, k) for k,v in list.items()]
        li.sort(reverse=True)
        for i in li:
            if l1sum <= l2sum:
                l1[i[1]] = i[0]
                l1sum += i[0]
            else:
                l2[i[1]] = i[0]
                l2sum += i[0]
        if (bottom_right.x - top_left.x) >= (bottom_right.y - top_left.y):
            x = int(float(l1sum) / float(l1sum + l2sum) * float(bottom_right.x - top_left.x)) + top_left.x
            if len(l1) > 1:
                self.divideDisplayArea(window, top_left, wx.Size(x, bottom_right.y), l1)
            else:
                # Buttons smaller than self.min_size must not be created
                if len(l1) == 1:
                    if x - top_left.x > self.min_size and bottom_right.y-top_left.y > self.min_size:
                        button = wx.Button(window, -1,
                                           l1.items()[0][0]+ ": " + str(l1.items()[0][1]),
                                           wx.Point(top_left.x, top_left.y),
                                           wx.Size(x - top_left.x,
                                                   bottom_right.y-top_left.y))
                        button.SetToolTipString(l1.items()[0][0]+ ": " + str(l1.items()[0][1]))
                        button.SetBackgroundColour('#7387e8')
                        self.button_id[l1.items()[0][0]] = button.GetId()
                        button.Bind(wx.EVT_BUTTON, self.onSelect)
                    else:
#                        print "Button for " + l1.items()[0][0]+ ": " + str(l1.items()[0][1]) + " not displayed " + str(wx.Size(x - top_left.x,
#                                                   bottom_right.y-top_left.y)) + "."
                        pass
            if len(l2) > 1:
                self.divideDisplayArea(window, wx.Size(x, top_left.y), bottom_right, l2)
            else:
                if len(l2) == 1:
                    if bottom_right.x - x > self.min_size and bottom_right.y-top_left.y > self.min_size:
                        button = wx.Button(window, -1,
                                           l2.items()[0][0]+ ": " + str(l2.items()[0][1]),
                                           wx.Point(x, top_left.y),
                                           wx.Size(bottom_right.x - x,
                                                   bottom_right.y-top_left.y))
                        button.SetToolTipString(l2.items()[0][0]+ ": " + str(l2.items()[0][1]))
                        button.SetBackgroundColour('#7387e8')
                        self.button_id[l2.items()[0][0]] = button.GetId()
                        button.Bind(wx.EVT_BUTTON, self.onSelect)
                    else:
#                        print "Button for " + l2.items()[0][0]+ ": " + str(l2.items()[0][1]) + " not displayed " + str(wx.Size(bottom_right.x - x,
#                                                   bottom_right.y-top_left.y)) + "."
                        pass
        else:
            y = int(float(l1sum) / float(l1sum + l2sum) * float(bottom_right.y - top_left.y)) + top_left.y
            if len(l1) > 1:
                self.divideDisplayArea(window, top_left, wx.Size(bottom_right.x, y), l1)
            else:
                if len(l1) == 1:
                    if bottom_right.x - top_left.x > self.min_size and y-top_left.y > self.min_size:
                        button = wx.Button(window, -1,
                                           l1.items()[0][0]+ ": " + str(l1.items()[0][1]),
                                           wx.Point(top_left.x, top_left.y),
                                           wx.Size(bottom_right.x - top_left.x,
                                                   y-top_left.y)) 
                        button.SetToolTipString(l1.items()[0][0]+ ": " + str(l1.items()[0][1]))
                        button.SetBackgroundColour('#7387e8')
                        self.button_id[l1.items()[0][0]] = button.GetId()
                        button.Bind(wx.EVT_BUTTON, self.onSelect)
                    else:
#                        print "Button for " + l1.items()[0][0]+ ": " + str(l1.items()[0][1]) + " not displayed " + str(wx.Size(bottom_right.x - top_left.x,
#                                                                                                                               y-top_left.y)) + "."
                        pass
            if len(l2) > 1:
                self.divideDisplayArea(window, wx.Size(top_left.x, y), bottom_right, l2)
            else:
                if len(l2) == 1:
                    if bottom_right.x - top_left.x > self.min_size and bottom_right.y-y > self.min_size:
                        button = wx.Button(window, -1,
                                           l2.items()[0][0]+ ": " + str(l2.items()[0][1]),
                                           wx.Point(top_left.x, y),
                                           wx.Size(bottom_right.x - top_left.x,
                                                   bottom_right.y-y))
                        button.SetToolTipString(l2.items()[0][0]+ ": " + str(l2.items()[0][1]))
                        button.SetBackgroundColour('#7387e8')
                        self.button_id[l2.items()[0][0]] = button.GetId()
                        button.Bind(wx.EVT_BUTTON, self.onSelect)
                    else:
#                        print "Button for " + l2.items()[0][0]+ ": " + str(l2.items()[0][1]) + " not displayed " + str(wx.Size(bottom_right.x - top_left.x,
#                                                   bottom_right.y-y)) + "."
                        pass


    def updateStatusBar (self):
        if len(self.filter) > 0:
            l = []
            for i in self.filter:
                l.append(i[0] + "=" + i[1])
            self.SetStatusText("Filter: " + ', '.join(l))
        else:
            self.SetStatusText("Filter: None")


    def updateChoice (self):
        # Disable the choice that cannot be used
        en_opt = alertgrp.fields[:]
        for i in self.filter:
            en_opt.remove(i[0])
        self.choice.SetItems(en_opt)


class AlertDsp (wx.App):
    """Alert Display application"""

    def OnInit(self):
        frame = AlertDspFrm("RaDaJo Alert Display",
                            (20, 40), (600,400))
        if len(sys.argv) > 1:
            frame.setData(sys.argv[1])
        frame.Show()
        self.SetTopWindow(frame)
        return True



if __name__ == "__main__":
    app = AlertDsp(False)
    app.MainLoop()
